/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_CACHE_BUFFER_H_
#define __GDAM_CACHE_BUFFER_H_

#include <gtk/gtkobject.h>

typedef struct _GdamCacheBuffer GdamCacheBuffer;

#include "gdamsource.h"
#include "gdamconnection.h"

GtkType gdam_cache_buffer_get_type ();
#define GDAM_TYPE_CACHE_BUFFER              (gdam_cache_buffer_get_type ())
#define GDAM_CACHE_BUFFER(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_CACHE_BUFFER, GdamCacheBuffer))

guint gdam_cache_buffer_get_done_event();
#define GDAM_CACHE_BUFFER_EVENT_DONE		(gdam_cache_buffer_get_done_event())

struct _GdamCacheBuffer {
	GtkObject		object;
	GdamSource*		to_drain;

	/* buffer to use while draining. */
	GdamBuffer		loading_buffer;

	/* after draining, copy the buffer into a contiguous block. */
	int			buffer_length;
	GdamSample*		buffer;
	GDestroyNotify          free_func;

	GdamConnection*		owner;

	/* our identifying number */
	guint			buffer_id;

	guint			source_id;	/* in gmainloop */
};

/* Call `new' then `register' to avoid memory leaks, as long
 * as you unregister it. */
GdamCacheBuffer*  gdam_cache_buffer_new       (GdamSource*      to_drain);

/* Don't call `gtk_object_unref' on the return value when you're done. */
GdamCacheBuffer*  gdam_cache_buffer_get_by_id (guint            buffer_id);

/* profile_mode is from gdam-protocol.h: enum GdamProtocolProfileMode. */
gint*             gdam_cache_buffer_profile   (GdamCacheBuffer* buffer,
				               int              profile_mode,
                                               int              sample_start,
				               int              sample_length,
				               int              num_outputs);

GdamCacheBuffer*  gdam_cache_buffer_memory    (int              buffer_length,
                                               GdamSample*      buffer,
					       GDestroyNotify   free_func);

#endif
