/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_CONNECTION_H_
#define __GDAM_CONNECTION_H_

#include <gdam/gdambuffer.h>
#include <gtk/gtkobject.h>
#include "gdamauthenticator.h"

/*
 * Describes a connection from a GDAM-client.
 */

typedef struct _GdamConnection GdamConnection;

struct _GdamConnection {
	/* Raw input from the client, waiting to gather enough
	 * data to be processed. */
	GdamBuffer 		client_commands;

	/* For authentify the user and gathering permissions. */
	GdamAuthenticator*	authenticator;

	/* For testing read/writability of the `from_client' channel. */
	GPollFD			client_poll;

	/* Our callback id from the GMainLoop,
	 * which we will need to remove to cleanup... */
	guint			source_id;

	/* Current permissions--
	 *
	 * If the authenticator still exists, the permissions
	 * should grant no rights-- it is the authenticator's
	 * job to endow them.
	 *
	 * If the authenticator is gone, then there is no
	 * way to change your permissions...
	 */
	GdamPermissions		permissions;

	/* List of responses (in GdamBuffers) from the server. */
	GdamBuffer		outgoing;

	/* Buffers we must destroy when we exit. */
	//GHashTable*		tmp_buffers;

	/* Temporary references to destroy on exit. */
	GHashTable*		ref_table;

	/* List of events the connection is subscribed to. */

	unsigned		is_readable : 1;
};

/* When we are setting/getting arguments/calls we set this
 * so that returned buffers can have "owners", when needed. */
extern GdamConnection* gdam_connection_for_arg;

GdamConnection*      gdam_connection_new                (int    client_fd,
                                                         GdamAuthenticator*);

/* The gdam-connection will be killed automatically by itself 
 * when it ends or commits a protocol violation. */
void                 gdam_connection_attach_to_main_loop(GdamConnection*);

/* Kill a connection (works regardless of if it is attached to
 *                    main_loop or not...) */
void                 gdam_connection_delete             (GdamConnection*);


/* For event emission code-- call this with a buffer which *does not*
 * have a length prepended...  The buffer will always be drained. */
void                 gdam_connection_emit_event         (GdamConnection*,
                                                         GdamBuffer* event);

gboolean	     gdam_connection_ref                (GdamConnection*,
                                                         GtkObject* object);
gboolean             gdam_connection_unref              (GdamConnection*,
                                                         GtkObject* object);
#endif
