/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_EXTERNAL_H_
#define __GDAM_EXTERNAL_H_

typedef struct _GdamExternal GdamExternal;
typedef struct _GdamExternalClass GdamExternalClass ;
GtkType gdam_external_get_type();
#define GDAM_TYPE_EXTERNAL   gdam_external_get_type()
#define GDAM_EXTERNAL(ptr) ((GdamExternal*) gtk_type_check_object_cast ((GtkTypeObject*) (ptr), GDAM_TYPE_EXTERNAL))
#define GDAM_EXTERNAL_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_EXTERNAL, GdamExternalClass))

struct _GdamExternalClass { 
	GdamSourceClass		source_class;
	void                  (*do_exec)(GdamExternal* external,
	                                 char**        command);
};

struct _GdamExternal {
	GdamSource		base;

	/* The command we execvp. */
	char**			command;

	/* The process id of the thread we started. */
	int			process_id;

	/* The file descriptor we're reading from. */
	GPollFD			poll_fd;

	/* The source-id in the GMainLoop. */
	guint			reader_source;

	/* Data buffered from the external source. */
	GdamBuffer		buffer;

	/* Maximum of data to buffer from source. */
	int			max_buffer_size;

	/* Data we must discard b/c we were behind. */
	int			skipped;

	gboolean		has_poll_fd;

	/*
	 * The next sample we are about to output.
	 * (So this is incremented entirely by `read' and `miss_data').
	 * (And zeroed by run_command and kill_command.)
	 */
	GdamTime		samples_output;

	/* Number of samples remaining to output, or -1 for no maximum. */
	GdamTime		samples_remaining;

	/* How many silent samples to output. */
	int			zero_time;

	unsigned		has_been_ready : 1;
};


GdamExternal*  gdam_external_new          (const char*   sh_command);

/* The command you pass is held by the external. */
gboolean       gdam_external_run_command  (GdamExternal* external, 
                                           char**        command);

gboolean       gdam_external_kill_command (GdamExternal* external,
                                           int*          exit_status);

int gdam_external_get_default_buffer_size();
void gdam_external_set_default_buffer_size(int buffer_size);

#endif
